# src/onako/worktree_manager.py
import subprocess
from pathlib import Path


class WorktreeManager:
    def __init__(self, repo_path: Path | str):
        self.repo_path = Path(repo_path).resolve()

    def _worktree_path(self, branch: str) -> Path:
        """Sibling directory with double-dash separator."""
        safe_branch = branch.replace("/", "--")
        return self.repo_path.parent / f"{self.repo_path.name}--{safe_branch}"

    def ensure_worktree(self, branch: str, base: str = "main") -> Path:
        """Ensure worktree exists for branch, creating if needed. Returns path."""
        wt_path = self._worktree_path(branch)

        # Check if worktree already exists
        if wt_path.exists():
            return wt_path

        # Check if branch exists
        result = subprocess.run(
            ["git", "branch", "--list", branch],
            cwd=self.repo_path, capture_output=True, text=True
        )
        branch_exists = branch in result.stdout

        if branch_exists:
            # Add worktree for existing branch
            subprocess.run(
                ["git", "worktree", "add", str(wt_path), branch],
                cwd=self.repo_path, check=True, capture_output=True
            )
        else:
            # Create new branch and worktree
            subprocess.run(
                ["git", "worktree", "add", "-b", branch, str(wt_path), base],
                cwd=self.repo_path, check=True, capture_output=True
            )

        return wt_path

    def list_worktrees(self) -> list[dict]:
        """List all worktrees with branch and path info."""
        result = subprocess.run(
            ["git", "worktree", "list", "--porcelain"],
            cwd=self.repo_path, capture_output=True, text=True, check=True
        )

        worktrees = []
        current = {}
        for line in result.stdout.strip().split("\n"):
            if line.startswith("worktree "):
                if current and current.get("branch"):
                    worktrees.append(current)
                current = {"path": Path(line[9:])}
            elif line.startswith("branch refs/heads/"):
                current["branch"] = line[18:]

        if current and current.get("branch"):
            worktrees.append(current)

        # Exclude main repo (it's a worktree too but not one we created)
        worktrees = [wt for wt in worktrees if wt["path"] != self.repo_path]

        return worktrees

    def delete_worktree(self, branch: str) -> None:
        """Remove worktree for the given branch."""
        wt_path = self._worktree_path(branch)

        subprocess.run(
            ["git", "worktree", "remove", "--force", str(wt_path)],
            cwd=self.repo_path, check=True, capture_output=True
        )



import os
import subprocess

from fastapi import FastAPI, HTTPException
from fastapi.responses import FileResponse
from fastapi.staticfiles import StaticFiles
from pydantic import BaseModel
from onako.tmux_orchestrator import TmuxOrchestrator
from onako.worktree_manager import WorktreeManager

app = FastAPI()
session_name = os.environ.get("ONAKO_SESSION", "onako")
skip_permissions_default = os.environ.get("ONAKO_SKIP_PERMISSIONS") == "1"
orch = TmuxOrchestrator(session_name=session_name)
static_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "static")

# Initialize worktree manager only if in a git repo
worktree_manager = WorktreeManager(os.getcwd()) if os.path.exists(os.path.join(os.getcwd(), ".git")) else None


class CreateTaskRequest(BaseModel):
    prompt: str
    working_dir: str | None = None
    skip_permissions: bool | None = None
    branch: str | None = None
    base_branch: str | None = None


class SendMessageRequest(BaseModel):
    message: str


@app.get("/")
def dashboard():
    return FileResponse(os.path.join(static_dir, "index.html"))


@app.get("/health")
def health():
    return {"status": "ok", "skip_permissions": skip_permissions_default}


@app.post("/tasks")
def create_task(req: CreateTaskRequest):
    skip = req.skip_permissions if req.skip_permissions is not None else skip_permissions_default
    from onako.task_service import create_task as _create_task
    try:
        return _create_task(
            orch, prompt=req.prompt, working_dir=req.working_dir, skip_permissions=skip,
            branch=req.branch, base_branch=req.base_branch, worktree_manager=worktree_manager,
        )
    except subprocess.CalledProcessError as e:
        raise HTTPException(status_code=400, detail=f"Failed to create worktree: {e.stderr.decode() if e.stderr else str(e)}")


@app.get("/tasks")
def list_tasks():
    return orch.list_tasks()


@app.get("/tasks/{task_id}")
def get_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    task = orch.tasks[task_id].copy()
    if task["status"] in ("done", "hidden") and task.get("last_output"):
        task["output"] = task["last_output"]
    else:
        task["output"] = orch.get_output(task_id)
    return task


@app.get("/tasks/{task_id}/raw")
def get_task_raw(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    task = orch.tasks[task_id].copy()
    task["output"] = orch.get_raw_output(task_id)
    return task


@app.post("/tasks/{task_id}/message")
def send_message(task_id: str, req: SendMessageRequest):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    orch.send_message(task_id, req.message)
    return {"status": "sent"}


@app.post("/tasks/{task_id}/interrupt")
def interrupt_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    orch.send_interrupt(task_id)
    return {"status": "interrupted"}


@app.post("/tasks/{task_id}/resume")
def resume_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    try:
        task = orch.resume_task(task_id, skip_permissions=skip_permissions_default, worktree_manager=worktree_manager)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    return task


@app.post("/tasks/{task_id}/hide")
def hide_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    try:
        orch.hide_task(task_id)
    except ValueError as e:
        raise HTTPException(status_code=400, detail=str(e))
    return orch.tasks[task_id]


@app.delete("/tasks/{task_id}")
def delete_task(task_id: str):
    if task_id not in orch.tasks:
        raise HTTPException(status_code=404, detail="Task not found")
    if orch.window_count() <= 1:
        raise HTTPException(status_code=400, detail="Cannot kill the last window")
    orch.kill_task(task_id)
    return {"status": "deleted"}


@app.get("/worktrees")
def list_worktrees():
    if not worktree_manager:
        return []
    return [{"branch": wt["branch"], "path": str(wt["path"])} for wt in worktree_manager.list_worktrees()]


@app.delete("/worktrees/{branch:path}")
def delete_worktree(branch: str):
    if not worktree_manager:
        raise HTTPException(status_code=400, detail="Not in a git repository")
    try:
        worktree_manager.delete_worktree(branch)
    except subprocess.CalledProcessError as e:
        detail = e.stderr.strip() if e.stderr else f"Failed to remove worktree for branch '{branch}'"
        raise HTTPException(status_code=400, detail=detail)
    return {"status": "deleted"}


@app.get("/stats")
def get_stats():
    return orch.get_stats()


app.mount("/static", StaticFiles(directory=static_dir), name="static")



import re
import secrets
import shlex
import sqlite3
import subprocess
import uuid
from datetime import datetime, timezone
from pathlib import Path


DB_PATH = Path.home() / ".onako" / "onako.db"


class TmuxOrchestrator:
    def __init__(self, session_name: str = "onako", db_path: Path | None = None):
        self.session_name = session_name
        self.db_path = db_path or DB_PATH
        self.tasks: dict[str, dict] = {}
        self._window_ids: dict[str, str] = {}  # window_name -> @id
        self._last_content_length: dict[str, int] = {}  # window_name -> pane content length
        self._last_output_cache: dict[str, str] = {}  # window_name -> cached stripped output
        self._init_db()
        self._load_tasks()
        self._ensure_session()
        self.rediscover_tasks()

    @property
    def _session_target(self) -> str:
        """Session name with colon suffix for unambiguous tmux targeting."""
        return f"{self.session_name}:"

    def _ensure_session(self):
        result = subprocess.run(
            ["tmux", "has-session", "-t", self._session_target],
            capture_output=True,
        )
        if result.returncode != 0:
            subprocess.run(
                ["tmux", "new-session", "-d", "-s", self.session_name, "-n", "onako-main"],
                check=True,
            )

    def _init_db(self):
        self.db_path.parent.mkdir(parents=True, exist_ok=True)
        conn = sqlite3.connect(self.db_path)
        conn.execute("""
            CREATE TABLE IF NOT EXISTS tasks (
                id TEXT PRIMARY KEY,
                prompt TEXT,
                status TEXT,
                started_at TEXT,
                origin TEXT DEFAULT 'managed',
                last_activity TEXT,
                claude_session_id TEXT,
                branch TEXT,
                worktree_path TEXT
            )
        """)
        # Migrate existing DBs that lack newer columns
        cursor = conn.execute("PRAGMA table_info(tasks)")
        columns = [row[1] for row in cursor.fetchall()]
        if "origin" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN origin TEXT DEFAULT 'managed'")
        if "last_activity" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN last_activity TEXT")
        if "claude_session_id" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN claude_session_id TEXT")
        if "branch" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN branch TEXT")
        if "worktree_path" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN worktree_path TEXT")
        if "last_output" not in columns:
            conn.execute("ALTER TABLE tasks ADD COLUMN last_output TEXT")
        conn.commit()

        # Events table for stats tracking
        conn.execute("""
            CREATE TABLE IF NOT EXISTS events (
                id INTEGER PRIMARY KEY AUTOINCREMENT,
                event_type TEXT NOT NULL,
                task_id TEXT,
                timestamp TEXT NOT NULL,
                metadata TEXT
            )
        """)

        # Backfill events from existing tasks (only if events table is empty)
        event_count = conn.execute("SELECT COUNT(*) FROM events").fetchone()[0]
        if event_count == 0:
            tasks = conn.execute("SELECT id, status, started_at, last_activity FROM tasks").fetchall()
            for task_id, status, started_at, last_activity in tasks:
                # Every task gets a task_created event
                conn.execute(
                    "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                    ("task_created", task_id, started_at or datetime.now(timezone.utc).isoformat()),
                )
                # Completed tasks get a task_completed event
                if status in ("done", "hidden"):
                    conn.execute(
                        "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                        ("task_completed", task_id, last_activity or started_at or datetime.now(timezone.utc).isoformat()),
                    )

        conn.commit()
        conn.close()

    def _load_tasks(self):
        conn = sqlite3.connect(self.db_path)
        rows = conn.execute("SELECT id, prompt, status, started_at, origin, last_activity, claude_session_id, branch, worktree_path, last_output FROM tasks").fetchall()
        conn.close()
        for row in rows:
            self.tasks[row[0]] = {
                "id": row[0],
                "prompt": row[1],
                "status": row[2],
                "started_at": row[3],
                "origin": row[4] or "managed",
                "last_activity": row[5],
                "claude_session_id": row[6],
                "branch": row[7],
                "worktree_path": row[8],
                "last_output": row[9],
            }

    def _save_task(self, task: dict):
        conn = sqlite3.connect(self.db_path)
        conn.execute(
            "INSERT OR REPLACE INTO tasks (id, prompt, status, started_at, origin, last_activity, claude_session_id, branch, worktree_path, last_output) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
            (task["id"], task["prompt"], task["status"], task["started_at"], task.get("origin", "managed"), task.get("last_activity"), task.get("claude_session_id"), task.get("branch"), task.get("worktree_path"), task.get("last_output")),
        )
        conn.commit()
        conn.close()

    def _log_event(self, event_type: str, task_id: str | None = None, metadata: str | None = None):
        conn = sqlite3.connect(self.db_path)
        conn.execute(
            "INSERT INTO events (event_type, task_id, timestamp, metadata) VALUES (?, ?, ?, ?)",
            (event_type, task_id, datetime.now(timezone.utc).isoformat(), metadata),
        )
        conn.commit()
        conn.close()

    def _run_tmux(self, *args) -> subprocess.CompletedProcess:
        return subprocess.run(["tmux", *args], capture_output=True, text=True)

    def _task_target(self, task_id: str) -> str:
        """Return a safe tmux target for a task, using window ID when available."""
        window_id = self._window_ids.get(task_id)
        if window_id:
            return window_id
        return f"{self.session_name}:{task_id}"

    def create_task(self, command: str, working_dir: str | None = None, prompt: str | None = None, claude_session_id: str | None = None, branch: str | None = None, worktree_path: str | None = None) -> dict:
        self._ensure_session()
        task_id = f"task-{secrets.token_hex(4)}"
        self._run_tmux(
            "new-window", "-t", self._session_target, "-n", task_id,
        )
        # Look up the window ID for safe targeting
        result = self._run_tmux(
            "list-windows", "-t", self._session_target, "-F", "#{window_name}|#{window_id}",
        )
        if result.stdout.strip():
            for line in result.stdout.strip().split("\n"):
                name, _, wid = line.partition("|")
                if name == task_id and wid:
                    self._window_ids[task_id] = wid
                    break
        if working_dir:
            self._run_tmux(
                "send-keys", "-t", self._task_target(task_id),
                f"cd {shlex.quote(working_dir)}", "Enter",
            )
        self._run_tmux(
            "send-keys", "-t", self._task_target(task_id),
            "-l", command,
        )
        self._run_tmux(
            "send-keys", "-t", self._task_target(task_id),
            "Enter",
        )
        now = datetime.now(timezone.utc).isoformat()
        task = {
            "id": task_id,
            "prompt": prompt or command,
            "status": "running",
            "started_at": now,
            "origin": "managed",
            "last_activity": now,
            "claude_session_id": claude_session_id,
            "branch": branch,
            "worktree_path": worktree_path,
        }
        self.tasks[task_id] = task
        self._save_task(task)
        self._log_event("task_created", task_id)
        return task

    def resume_task(self, task_id: str, skip_permissions: bool = False, worktree_manager=None) -> dict:
        """Resume a done task by reopening its tmux window."""
        task = self.tasks.get(task_id)
        if not task:
            raise ValueError("Task not found")
        if task["status"] == "running":
            raise ValueError("Task is still running")
        session_id = task.get("claude_session_id")
        if not session_id:
            raise ValueError("Task has no claude_session_id (cannot resume)")
        # Re-create worktree if it was cleaned up
        if task.get("worktree_path") and not Path(task["worktree_path"]).exists():
            if worktree_manager and task.get("branch"):
                worktree_manager.ensure_worktree(task["branch"])
        cmd = "claude"
        if skip_permissions:
            cmd += " --dangerously-skip-permissions"
        cmd += f" --resume {session_id}"
        # Create a new tmux window with the original task name
        self._ensure_session()
        self._run_tmux("new-window", "-t", self._session_target, "-n", task_id)
        # Look up the window ID for safe targeting
        result = self._run_tmux(
            "list-windows", "-t", self._session_target, "-F", "#{window_name}|#{window_id}",
        )
        if result.stdout.strip():
            for line in result.stdout.strip().split("\n"):
                name, _, wid = line.partition("|")
                if name == task_id and wid:
                    self._window_ids[task_id] = wid
                    break
        # cd to worktree if task was running in one
        if task.get("worktree_path"):
            self._run_tmux(
                "send-keys", "-t", self._task_target(task_id),
                f"cd {shlex.quote(task['worktree_path'])}", "Enter",
            )
        self._run_tmux("send-keys", "-t", self._task_target(task_id), "-l", cmd)
        self._run_tmux("send-keys", "-t", self._task_target(task_id), "Enter")
        now = datetime.now(timezone.utc).isoformat()
        task["status"] = "running"
        task["last_output"] = None
        task["last_activity"] = now
        if not task["prompt"].startswith("(resumed)"):
            task["prompt"] = f"(resumed) {task['prompt']}"
        self._save_task(task)
        self._log_event("task_resumed", task_id)
        return task

    def list_tasks(self) -> list[dict]:
        self.rediscover_tasks()
        self._sync_task_status()
        return list(self.tasks.values())

    _ANSI_RE = re.compile(r"\x1b\[[0-9;]*[a-zA-Z]")

    def get_output(self, task_id: str) -> str:
        raw = self.get_raw_output(task_id)
        return self._strip_claude_chrome(raw)

    @classmethod
    def _strip_claude_chrome(cls, text: str) -> str:
        lines = text.split("\n")
        # Strip from the bottom: Claude Code's TUI draws box-drawing chars,
        # the › prompt, and status lines like "accept edits on..."
        while lines:
            line = cls._ANSI_RE.sub("", lines[-1]).strip()
            if (
                not line
                or all(c in "─━╭╮╰╯│┃┌┐└┘├┤┬┴┼╋═║ ›❯▸▶" for c in line)
                or "accept edits" in line
                or "esc to interrupt" in line
                or "shift+tab to cycle" in line
                or "ctrl+" in line.lower()
                or line == "›"
            ):
                lines.pop()
            else:
                break
        return "\n".join(lines)

    def get_raw_output(self, task_id: str) -> str:
        result = self._run_tmux(
            "capture-pane", "-t", self._task_target(task_id),
            "-p", "-S", "-", "-e",
        )
        return result.stdout

    def send_message(self, task_id: str, message: str):
        self._run_tmux(
            "send-keys", "-t", self._task_target(task_id),
            "-l", message,
        )
        self._run_tmux(
            "send-keys", "-t", self._task_target(task_id),
            "Enter",
        )
        self._log_event("message_sent", task_id)

    def send_interrupt(self, task_id: str):
        self._run_tmux("send-keys", "-t", self._task_target(task_id), "Escape")
        self._log_event("task_interrupted", task_id)

    def window_count(self) -> int:
        result = self._run_tmux(
            "list-windows", "-t", self._session_target, "-F", "#{window_name}",
        )
        if result.stdout.strip():
            return len(result.stdout.strip().split("\n"))
        return 0

    def kill_task(self, task_id: str):
        # Snapshot output before killing the window
        try:
            output = self.get_output(task_id)
            lines = output.split("\n")
            snapshot = "\n".join(lines[-200:]) if len(lines) > 200 else output
        except Exception:
            snapshot = None
        self._run_tmux("kill-window", "-t", self._task_target(task_id))
        if task_id in self.tasks:
            self.tasks[task_id]["status"] = "done"
            if snapshot:
                self.tasks[task_id]["last_output"] = snapshot
            self._save_task(self.tasks[task_id])
            self._log_event("task_killed", task_id)

    def hide_task(self, task_id: str):
        """Hide a done task from the dashboard."""
        task = self.tasks.get(task_id)
        if not task:
            raise ValueError("Task not found")
        if task["status"] != "done":
            raise ValueError("Task is not done")
        task["status"] = "hidden"
        self._save_task(task)

    def _sync_task_status(self):
        result = self._run_tmux(
            "list-windows", "-t", self._session_target, "-F", "#{window_name}|#{window_id}",
        )
        active_windows = set()
        if result.stdout.strip():
            for line in result.stdout.strip().split("\n"):
                parts = line.split("|", 1)
                active_windows.add(parts[0])
                if len(parts) > 1:
                    self._window_ids[parts[0]] = parts[1]
        for task_id, task in self.tasks.items():
            if task["status"] == "running" and task_id not in active_windows:
                task["status"] = "done"
                cached = self._last_output_cache.pop(task_id, None)
                if cached:
                    task["last_output"] = cached
                self._save_task(task)
                self._log_event("task_completed", task_id)
            elif task["status"] == "running" and task_id in active_windows:
                pane = self._run_tmux(
                    "capture-pane", "-t", self._task_target(task_id), "-p", "-S", "-", "-e",
                )
                raw = pane.stdout
                content_len = len(raw)
                prev_len = self._last_content_length.get(task_id)
                self._last_content_length[task_id] = content_len
                # Cache stripped output for snapshot on natural finish
                stripped = self._strip_claude_chrome(raw)
                lines = stripped.split("\n")
                self._last_output_cache[task_id] = "\n".join(lines[-200:]) if len(lines) > 200 else stripped
                if prev_len is not None and content_len != prev_len:
                    task["last_activity"] = datetime.now(timezone.utc).isoformat()
                    self._save_task(task)

    def get_stats(self) -> dict:
        """Return aggregated stats from the events table."""
        conn = sqlite3.connect(self.db_path)
        conn.row_factory = sqlite3.Row

        # --- Volume ---
        counts = conn.execute("""
            SELECT
                COALESCE(SUM(CASE WHEN event_type = 'task_created' THEN 1 ELSE 0 END), 0) AS total_tasks,
                COALESCE(SUM(CASE WHEN event_type = 'message_sent' THEN 1 ELSE 0 END), 0) AS total_messages_sent,
                COALESCE(SUM(CASE WHEN event_type = 'task_resumed' THEN 1 ELSE 0 END), 0) AS total_resumes,
                COALESCE(SUM(CASE WHEN event_type = 'task_killed' THEN 1 ELSE 0 END), 0) AS total_kills,
                COALESCE(SUM(CASE WHEN event_type = 'task_interrupted' THEN 1 ELSE 0 END), 0) AS total_interrupts
            FROM events
        """).fetchone()

        # Runtime: sum durations from task_created to task_completed (or now) per task
        created_rows = conn.execute(
            "SELECT task_id, timestamp FROM events WHERE event_type = 'task_created'"
        ).fetchall()
        completed_rows = conn.execute(
            "SELECT task_id, timestamp FROM events WHERE event_type = 'task_completed'"
        ).fetchall()
        completed_map: dict[str, str] = {}
        for row in completed_rows:
            completed_map[row["task_id"]] = row["timestamp"]

        now = datetime.now(timezone.utc)
        total_runtime = 0
        for row in created_rows:
            task_id = row["task_id"]
            start = datetime.fromisoformat(row["timestamp"])
            if task_id in completed_map:
                end = datetime.fromisoformat(completed_map[task_id])
            else:
                end = now
            total_runtime += int((end - start).total_seconds())

        volume = {
            "total_tasks": counts["total_tasks"],
            "total_messages_sent": counts["total_messages_sent"],
            "total_resumes": counts["total_resumes"],
            "total_kills": counts["total_kills"],
            "total_interrupts": counts["total_interrupts"],
            "total_runtime_seconds": total_runtime,
        }

        # --- Records ---
        # Longest task: task with max duration between created and completed
        longest_task = None
        if created_rows and completed_map:
            best_id = None
            best_duration = -1
            for row in created_rows:
                tid = row["task_id"]
                if tid in completed_map:
                    start = datetime.fromisoformat(row["timestamp"])
                    end = datetime.fromisoformat(completed_map[tid])
                    dur = int((end - start).total_seconds())
                    if dur > best_duration:
                        best_duration = dur
                        best_id = tid
            if best_id is not None:
                prompt = self.tasks.get(best_id, {}).get("prompt", "")
                longest_task = {"id": best_id, "prompt": prompt, "duration_seconds": best_duration}

        # Most resumed task
        most_resumed_task = None
        resumed_row = conn.execute(
            "SELECT task_id, COUNT(*) as cnt FROM events WHERE event_type = 'task_resumed' GROUP BY task_id ORDER BY cnt DESC LIMIT 1"
        ).fetchone()
        if resumed_row:
            tid = resumed_row["task_id"]
            prompt = self.tasks.get(tid, {}).get("prompt", "")
            most_resumed_task = {"id": tid, "prompt": prompt, "resume_count": resumed_row["cnt"]}

        # Busiest day (most task_created events)
        busiest_day = None
        busiest_row = conn.execute(
            "SELECT SUBSTR(timestamp, 1, 10) AS date, COUNT(*) AS cnt FROM events WHERE event_type = 'task_created' GROUP BY date ORDER BY cnt DESC LIMIT 1"
        ).fetchone()
        if busiest_row:
            busiest_day = {"date": busiest_row["date"], "task_count": busiest_row["cnt"]}

        # Biggest day for messages
        biggest_day_messages = None
        biggest_msg_row = conn.execute(
            "SELECT SUBSTR(timestamp, 1, 10) AS date, COUNT(*) AS cnt FROM events WHERE event_type = 'message_sent' GROUP BY date ORDER BY cnt DESC LIMIT 1"
        ).fetchone()
        if biggest_msg_row:
            biggest_day_messages = {"date": biggest_msg_row["date"], "message_count": biggest_msg_row["cnt"]}

        records = {
            "longest_task": longest_task,
            "most_resumed_task": most_resumed_task,
            "busiest_day": busiest_day,
            "biggest_day_messages": biggest_day_messages,
        }

        # --- Trends (ISO week grouping, done in Python for correctness) ---
        task_created_rows = conn.execute(
            "SELECT timestamp FROM events WHERE event_type = 'task_created'"
        ).fetchall()
        message_sent_rows = conn.execute(
            "SELECT timestamp FROM events WHERE event_type = 'message_sent'"
        ).fetchall()

        tasks_by_week: dict[str, int] = {}
        for row in task_created_rows:
            dt = datetime.fromisoformat(row["timestamp"])
            iso_year, iso_week, _ = dt.isocalendar()
            week_key = f"{iso_year}-W{iso_week:02d}"
            tasks_by_week[week_key] = tasks_by_week.get(week_key, 0) + 1

        messages_by_week: dict[str, int] = {}
        for row in message_sent_rows:
            dt = datetime.fromisoformat(row["timestamp"])
            iso_year, iso_week, _ = dt.isocalendar()
            week_key = f"{iso_year}-W{iso_week:02d}"
            messages_by_week[week_key] = messages_by_week.get(week_key, 0) + 1

        trends = {
            "tasks_per_week": [{"week": w, "count": c} for w, c in sorted(tasks_by_week.items())],
            "messages_per_week": [{"week": w, "count": c} for w, c in sorted(messages_by_week.items())],
        }

        # --- Heatmap: event count per calendar day ---
        heatmap_rows = conn.execute(
            "SELECT SUBSTR(timestamp, 1, 10) AS date, COUNT(*) AS cnt FROM events GROUP BY date ORDER BY date"
        ).fetchall()
        heatmap = {row["date"]: row["cnt"] for row in heatmap_rows}

        # --- Activity by hour: 24-element array ---
        activity_by_hour = [0] * 24
        hour_rows = conn.execute("SELECT timestamp FROM events").fetchall()
        for row in hour_rows:
            dt = datetime.fromisoformat(row["timestamp"])
            activity_by_hour[dt.hour] += 1

        conn.close()

        return {
            "volume": volume,
            "records": records,
            "trends": trends,
            "heatmap": heatmap,
            "activity_by_hour": activity_by_hour,
        }

    def rediscover_tasks(self):
        """Rediscover tasks from existing tmux windows on server restart."""
        result = self._run_tmux(
            "list-windows", "-t", self._session_target, "-F", "#{window_name}|#{window_id}",
        )
        if not result.stdout.strip():
            return
        for line in result.stdout.strip().split("\n"):
            parts = line.split("|", 1)
            window_name = parts[0]
            window_id = parts[1] if len(parts) > 1 else None
            if window_id:
                self._window_ids[window_name] = window_id
            if window_name not in self.tasks:
                is_managed = window_name.startswith("task-") or window_name == "onako-main"
                now = datetime.now(timezone.utc).isoformat()
                task = {
                    "id": window_name,
                    "prompt": "(rediscovered)" if is_managed else window_name,
                    "status": "running",
                    "started_at": None,
                    "origin": "managed" if is_managed else "external",
                    "last_activity": now,
                    "branch": None,
                    "worktree_path": None,
                }
                self.tasks[window_name] = task
                self._save_task(task)
            else:
                changed = False
                if self.tasks[window_name]["status"] == "done":
                    self.tasks[window_name]["status"] = "running"
                    changed = True
                if window_name == "onako-main" and self.tasks[window_name]["origin"] == "external":
                    self.tasks[window_name]["origin"] = "managed"
                    changed = True
                if not self.tasks[window_name].get("last_activity"):
                    self.tasks[window_name]["last_activity"] = datetime.now(timezone.utc).isoformat()
                    changed = True
                if changed:
                    self._save_task(self.tasks[window_name])



import subprocess
import time
from pathlib import Path
import pytest
from onako.tmux_orchestrator import TmuxOrchestrator

SESSION_NAME = "onako-test"


@pytest.fixture(autouse=True)
def cleanup(tmp_path):
    """Kill test tmux session before and after each test."""
    subprocess.run(["tmux", "kill-session", "-t", SESSION_NAME], capture_output=True)
    yield
    subprocess.run(["tmux", "kill-session", "-t", SESSION_NAME], capture_output=True)


@pytest.fixture
def orch(tmp_path):
    return TmuxOrchestrator(session_name=SESSION_NAME, db_path=tmp_path / "test.db")


def test_create_task_returns_id(orch):
    task = orch.create_task("echo hello")
    assert task["id"].startswith("task-")
    assert task["status"] == "running"
    assert task["prompt"] == "echo hello"


def test_list_tasks_shows_created_task(orch):
    task = orch.create_task("echo hello")
    tasks = orch.list_tasks()
    assert any(t["id"] == task["id"] for t in tasks)


def test_get_output_captures_pane(orch):
    task = orch.create_task("echo hello-from-test")
    time.sleep(1)
    output = orch.get_output(task["id"])
    assert "hello-from-test" in output


def test_get_raw_output(orch):
    task = orch.create_task("echo raw-test")
    time.sleep(1)
    raw = orch.get_raw_output(task["id"])
    assert "raw-test" in raw


def test_send_message(orch):
    task = orch.create_task("cat")  # cat waits for stdin
    time.sleep(0.5)
    orch.send_message(task["id"], "hello-input")
    time.sleep(1)
    output = orch.get_output(task["id"])
    assert "hello-input" in output


def test_send_interrupt(orch):
    task = orch.create_task("sleep 999")
    time.sleep(0.5)
    orch.send_interrupt(task["id"])
    time.sleep(0.5)
    # Window should still exist after interrupt
    tasks = orch.list_tasks()
    assert any(t["id"] == task["id"] and t["status"] == "running" for t in tasks)


def test_kill_task(orch):
    task = orch.create_task("sleep 999")
    orch.kill_task(task["id"])
    tasks = orch.list_tasks()
    assert not any(t["id"] == task["id"] and t["status"] == "running" for t in tasks)


def test_create_multiple_tasks(orch):
    t1 = orch.create_task("echo one")
    t2 = orch.create_task("echo two")
    tasks = orch.list_tasks()
    ids = [t["id"] for t in tasks]
    assert t1["id"] in ids
    assert t2["id"] in ids
    assert t1["id"] != t2["id"]


def test_created_task_has_managed_origin(orch):
    task = orch.create_task("echo hello")
    assert task["origin"] == "managed"


def test_rediscover_external_window(orch):
    """Windows not created by onako are discovered with origin=external."""
    subprocess.run([
        "tmux", "new-window", "-t", SESSION_NAME, "-n", "my-feature",
    ], check=True)
    time.sleep(0.5)
    orch.rediscover_tasks()
    tasks = orch.list_tasks()
    external = [t for t in tasks if t["id"] == "my-feature"]
    assert len(external) == 1
    assert external[0]["origin"] == "external"


def test_external_window_output(orch):
    """Can read output from externally created windows."""
    subprocess.run([
        "tmux", "new-window", "-t", SESSION_NAME, "-n", "ext-test",
    ], check=True)
    subprocess.run([
        "tmux", "send-keys", "-t", f"{SESSION_NAME}:ext-test",
        "echo external-output", "Enter",
    ], check=True)
    time.sleep(1)
    orch.rediscover_tasks()
    output = orch.get_output("ext-test")
    assert "external-output" in output


def test_external_window_send_message(orch):
    """Can send messages to externally created windows."""
    subprocess.run([
        "tmux", "new-window", "-t", SESSION_NAME, "-n", "ext-msg",
    ], check=True)
    subprocess.run([
        "tmux", "send-keys", "-t", f"{SESSION_NAME}:ext-msg",
        "cat", "Enter",
    ], check=True)
    time.sleep(0.5)
    orch.rediscover_tasks()
    orch.send_message("ext-msg", "hello-external")
    time.sleep(1)
    output = orch.get_output("ext-msg")
    assert "hello-external" in output


def test_external_window_kill(orch):
    """Can kill externally created windows."""
    subprocess.run([
        "tmux", "new-window", "-t", SESSION_NAME, "-n", "ext-kill",
    ], check=True)
    time.sleep(0.5)
    orch.rediscover_tasks()
    orch.kill_task("ext-kill")
    tasks = orch.list_tasks()
    assert not any(t["id"] == "ext-kill" and t["status"] == "running" for t in tasks)


def test_hide_task(orch):
    """Hiding a done task sets status to 'hidden'."""
    task = orch.create_task("echo hide-me")
    orch.kill_task(task["id"])
    assert orch.tasks[task["id"]]["status"] == "done"
    orch.hide_task(task["id"])
    assert orch.tasks[task["id"]]["status"] == "hidden"


def test_hide_task_not_running(orch):
    """Cannot hide a running task."""
    task = orch.create_task("sleep 999")
    with pytest.raises(ValueError, match="not done"):
        orch.hide_task(task["id"])


def test_hide_task_not_found(orch):
    """Hiding a nonexistent task raises."""
    with pytest.raises(ValueError, match="not found"):
        orch.hide_task("task-nonexistent")


def test_resume_task_reuses_same_record(orch):
    """Resuming a task reuses the original record instead of creating a new one."""
    task = orch.create_task("echo original")
    task_id = task["id"]
    # Simulate a claude_session_id (normally set by server)
    orch.tasks[task_id]["claude_session_id"] = "fake-session-id"
    orch._save_task(orch.tasks[task_id])
    # Kill the task so it becomes "done"
    orch.kill_task(task_id)
    assert orch.tasks[task_id]["status"] == "done"
    # Resume
    resumed = orch.resume_task(task_id)
    assert resumed["id"] == task_id  # same record
    assert resumed["status"] == "running"
    assert "(resumed)" in resumed["prompt"]
    # No new task record was created
    task_ids = [t["id"] for t in orch.list_tasks() if "original" in t["prompt"]]
    assert task_ids == [task_id]


def test_create_task_with_branch(orch):
    """Tasks can be created with branch and worktree_path."""
    task = orch.create_task("echo hello", branch="feature-x", worktree_path="/tmp/fake")
    assert task["branch"] == "feature-x"
    assert task["worktree_path"] == "/tmp/fake"


def test_create_task_without_branch(orch):
    """Tasks created without branch have None values."""
    task = orch.create_task("echo hello")
    assert task.get("branch") is None
    assert task.get("worktree_path") is None


def test_kill_task_saves_last_output(orch):
    """Killing a task snapshots the last output."""
    task = orch.create_task("echo snapshot-test-content")
    time.sleep(1)
    orch.kill_task(task["id"])
    assert orch.tasks[task["id"]]["status"] == "done"
    assert "snapshot-test-content" in orch.tasks[task["id"]].get("last_output", "")


def test_resume_clears_last_output(orch):
    """Resuming a task clears the saved last_output."""
    task = orch.create_task("echo resume-clear-test")
    task_id = task["id"]
    orch.tasks[task_id]["claude_session_id"] = "fake-session-id"
    orch._save_task(orch.tasks[task_id])
    time.sleep(1)
    orch.kill_task(task_id)
    assert orch.tasks[task_id].get("last_output")  # has snapshot
    orch.resume_task(task_id)
    assert orch.tasks[task_id].get("last_output") is None


def test_natural_finish_saves_last_output(orch):
    """When a task finishes naturally (window closes), last_output is saved."""
    task = orch.create_task("echo natural-finish-content && sleep 1 && exit 0")
    time.sleep(0.5)  # let echo run but before exit
    orch.list_tasks()  # sync tick caches output while window is alive
    time.sleep(2)  # wait for exit 0 to close the window
    orch.list_tasks()  # sync tick detects window gone, persists cached output
    t = orch.tasks[task["id"]]
    assert t["status"] == "done"
    assert "natural-finish-content" in t.get("last_output", "")


def test_resume_recreates_missing_worktree(orch, tmp_path):
    """Resume re-creates a worktree if it was cleaned up."""
    from unittest.mock import MagicMock

    task = orch.create_task("echo hello", branch="feat/test", worktree_path=str(tmp_path / "gone"))
    task_id = task["id"]
    orch.tasks[task_id]["claude_session_id"] = "fake-session-id"
    orch._save_task(orch.tasks[task_id])
    orch.kill_task(task_id)

    # Worktree dir doesn't exist (was cleaned)
    assert not (tmp_path / "gone").exists()

    mock_wm = MagicMock()
    mock_wm.ensure_worktree.return_value = tmp_path / "gone"
    orch.resume_task(task_id, worktree_manager=mock_wm)
    mock_wm.ensure_worktree.assert_called_once_with("feat/test")



import sqlite3
from pathlib import Path
from unittest.mock import patch

import pytest


@pytest.fixture
def db_path(tmp_path):
    return tmp_path / "test.db"


@pytest.fixture
def make_orchestrator(db_path):
    """Factory that creates a TmuxOrchestrator with mocked tmux calls."""
    def _make():
        with patch("onako.tmux_orchestrator.TmuxOrchestrator._ensure_session"), \
             patch("onako.tmux_orchestrator.TmuxOrchestrator.rediscover_tasks"):
            from onako.tmux_orchestrator import TmuxOrchestrator
            orch = TmuxOrchestrator(session_name="test", db_path=db_path)
            return orch
    return _make


def test_events_table_created(make_orchestrator, db_path):
    """Events table should be created on init."""
    make_orchestrator()
    conn = sqlite3.connect(db_path)
    cursor = conn.execute("SELECT name FROM sqlite_master WHERE type='table' AND name='events'")
    assert cursor.fetchone() is not None
    conn.close()


def test_events_table_columns(make_orchestrator, db_path):
    """Events table should have correct columns."""
    make_orchestrator()
    conn = sqlite3.connect(db_path)
    cursor = conn.execute("PRAGMA table_info(events)")
    columns = {row[1] for row in cursor.fetchall()}
    assert columns == {"id", "event_type", "task_id", "timestamp", "metadata"}
    conn.close()


def test_backfill_creates_events_from_existing_tasks(db_path, make_orchestrator):
    """Backfill should create events from pre-existing tasks."""
    # Pre-populate tasks table
    conn = sqlite3.connect(db_path)
    conn.execute("""
        CREATE TABLE IF NOT EXISTS tasks (
            id TEXT PRIMARY KEY, prompt TEXT, status TEXT, started_at TEXT,
            origin TEXT DEFAULT 'managed', last_activity TEXT,
            claude_session_id TEXT, branch TEXT, worktree_path TEXT
        )
    """)
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-old1", "do stuff", "done", "2026-01-15T10:00:00+00:00", "managed",
         "2026-01-15T11:00:00+00:00", None, None, None),
    )
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-old2", "more stuff", "running", "2026-01-16T10:00:00+00:00", "managed",
         "2026-01-16T10:30:00+00:00", None, None, None),
    )
    conn.commit()
    conn.close()

    make_orchestrator()

    conn = sqlite3.connect(db_path)
    events = conn.execute("SELECT event_type, task_id, timestamp FROM events ORDER BY id").fetchall()
    conn.close()

    # task-old1: should have task_created + task_completed
    # task-old2: should have task_created only (still running)
    created = [e for e in events if e[0] == "task_created"]
    completed = [e for e in events if e[0] == "task_completed"]
    assert len(created) == 2
    assert len(completed) == 1
    assert completed[0][1] == "task-old1"


def test_backfill_skips_if_events_already_exist(db_path, make_orchestrator):
    """Backfill should not run if events table already has data."""
    # Pre-populate tasks and events
    conn = sqlite3.connect(db_path)
    conn.execute("""
        CREATE TABLE IF NOT EXISTS tasks (
            id TEXT PRIMARY KEY, prompt TEXT, status TEXT, started_at TEXT,
            origin TEXT DEFAULT 'managed', last_activity TEXT,
            claude_session_id TEXT, branch TEXT, worktree_path TEXT
        )
    """)
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-old1", "do stuff", "done", "2026-01-15T10:00:00+00:00", "managed",
         "2026-01-15T11:00:00+00:00", None, None, None),
    )
    conn.execute("""
        CREATE TABLE IF NOT EXISTS events (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            event_type TEXT NOT NULL, task_id TEXT,
            timestamp TEXT NOT NULL, metadata TEXT
        )
    """)
    conn.execute(
        "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
        ("task_created", "task-old1", "2026-01-15T10:00:00+00:00"),
    )
    conn.commit()
    conn.close()

    make_orchestrator()

    conn = sqlite3.connect(db_path)
    count = conn.execute("SELECT COUNT(*) FROM events").fetchone()[0]
    conn.close()
    # Should still be 1, no duplicates from backfill
    assert count == 1


def test_create_task_logs_event(make_orchestrator, db_path):
    """Creating a task should log a task_created event."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        orch.create_task(command="echo hi", prompt="test prompt")
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='task_created'"
    ).fetchall()
    conn.close()
    assert len(events) == 1
    assert events[0][1].startswith("task-")


def test_kill_task_logs_event(make_orchestrator, db_path):
    """Killing a task should log a task_killed event."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        task = orch.create_task(command="echo hi", prompt="test")
    with patch.object(orch, "_run_tmux"):
        orch.kill_task(task["id"])
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='task_killed'"
    ).fetchall()
    conn.close()
    assert len(events) == 1
    assert events[0][1] == task["id"]


def test_send_message_logs_event(make_orchestrator, db_path):
    """Sending a message should log a message_sent event."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        task = orch.create_task(command="echo hi", prompt="test")
    with patch.object(orch, "_run_tmux"):
        orch.send_message(task["id"], "hello")
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='message_sent'"
    ).fetchall()
    conn.close()
    assert len(events) == 1
    assert events[0][1] == task["id"]


def test_send_interrupt_logs_event(make_orchestrator, db_path):
    """Interrupting a task should log a task_interrupted event."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        task = orch.create_task(command="echo hi", prompt="test")
    with patch.object(orch, "_run_tmux"):
        orch.send_interrupt(task["id"])
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='task_interrupted'"
    ).fetchall()
    conn.close()
    assert len(events) == 1


def test_resume_task_logs_event(make_orchestrator, db_path):
    """Resuming a task should log a task_resumed event."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        task = orch.create_task(command="echo hi", prompt="test", claude_session_id="fake-sid")
    # Mark task as done so it can be resumed
    orch.tasks[task["id"]]["status"] = "done"
    orch._save_task(orch.tasks[task["id"]])
    with patch.object(orch, "_run_tmux"):
        orch.resume_task(task["id"])
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='task_resumed'"
    ).fetchall()
    conn.close()
    assert len(events) == 1


def test_task_completed_logged_on_status_change(make_orchestrator, db_path):
    """When _sync_task_status detects a task is done, it should log task_completed."""
    orch = make_orchestrator()
    with patch.object(orch, "_run_tmux"):
        task = orch.create_task(command="echo hi", prompt="test")
    # Simulate window disappearing (task completed)
    mock_result = type("R", (), {"stdout": "", "returncode": 0})()
    with patch.object(orch, "_run_tmux", return_value=mock_result):
        orch._sync_task_status()
    conn = sqlite3.connect(db_path)
    events = conn.execute(
        "SELECT event_type, task_id FROM events WHERE event_type='task_completed'"
    ).fetchall()
    conn.close()
    assert len(events) == 1
    assert events[0][1] == task["id"]



import subprocess
import pytest
from fastapi.testclient import TestClient

API_SESSION = "onako-api-test"


@pytest.fixture(autouse=True)
def cleanup_session():
    subprocess.run(["tmux", "kill-session", "-t", API_SESSION], capture_output=True)
    yield
    subprocess.run(["tmux", "kill-session", "-t", API_SESSION], capture_output=True)


@pytest.fixture
def client(tmp_path):
    import os
    import importlib
    os.environ["ONAKO_SESSION"] = API_SESSION
    from onako import tmux_orchestrator
    original_db = tmux_orchestrator.DB_PATH
    tmux_orchestrator.DB_PATH = tmp_path / "test.db"
    from onako import server
    importlib.reload(server)
    client = TestClient(server.app)
    yield client
    response = client.get("/tasks")
    for task in response.json():
        if task["id"] != "onako-main":
            client.delete(f"/tasks/{task['id']}")
    tmux_orchestrator.DB_PATH = original_db


def test_health(client):
    r = client.get("/health")
    assert r.status_code == 200
    assert r.json()["status"] == "ok"


def test_create_task(client):
    r = client.post("/tasks", json={"prompt": "echo api-test"})
    assert r.status_code == 200
    assert r.json()["id"].startswith("task-")


def test_list_tasks(client):
    client.post("/tasks", json={"prompt": "echo one"})
    client.post("/tasks", json={"prompt": "echo two"})
    r = client.get("/tasks")
    assert r.status_code == 200
    assert len(r.json()) >= 2


def test_get_task(client):
    create = client.post("/tasks", json={"prompt": "echo detail-test"})
    task_id = create.json()["id"]
    import time
    time.sleep(1)
    r = client.get(f"/tasks/{task_id}")
    assert r.status_code == 200
    assert r.json()["id"] == task_id
    assert "output" in r.json()


def test_get_task_raw(client):
    create = client.post("/tasks", json={"prompt": "echo raw-api-test"})
    task_id = create.json()["id"]
    import time
    time.sleep(1)
    r = client.get(f"/tasks/{task_id}/raw")
    assert r.status_code == 200
    assert "output" in r.json()


def test_send_message(client):
    create = client.post("/tasks", json={"prompt": "cat"})
    task_id = create.json()["id"]
    r = client.post(f"/tasks/{task_id}/message", json={"message": "hello"})
    assert r.status_code == 200


def test_delete_task(client):
    create = client.post("/tasks", json={"prompt": "sleep 999"})
    task_id = create.json()["id"]
    r = client.delete(f"/tasks/{task_id}")
    assert r.status_code == 200


def test_interrupt_task(client):
    create = client.post("/tasks", json={"prompt": "sleep 999"})
    task_id = create.json()["id"]
    r = client.post(f"/tasks/{task_id}/interrupt")
    assert r.status_code == 200
    assert r.json()["status"] == "interrupted"


def test_interrupt_nonexistent_task(client):
    r = client.post("/tasks/task-nonexistent/interrupt")
    assert r.status_code == 404


def test_get_nonexistent_task(client):
    r = client.get("/tasks/task-nonexistent")
    assert r.status_code == 404


def test_health_includes_skip_permissions(client):
    r = client.get("/health")
    assert r.status_code == 200
    assert "skip_permissions" in r.json()
    assert r.json()["skip_permissions"] is False


def test_create_task_default_no_skip(client):
    """When skip_permissions_default is False, command should not include the flag."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})
    with patch.object(server, "skip_permissions_default", False), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "echo test"})
        assert r.status_code == 200
        command = mock_create.call_args[1]["command"]
        assert command.startswith("claude --session-id ")
        assert command.endswith("'echo test'")
        assert "--dangerously-skip-permissions" not in command


def test_create_task_with_global_skip(client):
    """When skip_permissions_default is True, command should include the flag."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})
    with patch.object(server, "skip_permissions_default", True), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "echo test"})
        assert r.status_code == 200
        command = mock_create.call_args[1]["command"]
        assert command.startswith("claude --dangerously-skip-permissions --session-id ")
        assert command.endswith("'echo test'")


def test_create_task_per_task_skip_override(client):
    """Per-task skip_permissions=True overrides global default of False."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})
    with patch.object(server, "skip_permissions_default", False), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "echo test", "skip_permissions": True})
        assert r.status_code == 200
        command = mock_create.call_args[1]["command"]
        assert command.startswith("claude --dangerously-skip-permissions --session-id ")
        assert command.endswith("'echo test'")


def test_create_task_per_task_no_skip_override(client):
    """Per-task skip_permissions=False overrides global default of True."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})
    with patch.object(server, "skip_permissions_default", True), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "echo test", "skip_permissions": False})
        assert r.status_code == 200
        command = mock_create.call_args[1]["command"]
        assert command.startswith("claude --session-id ")
        assert command.endswith("'echo test'")
        assert "--dangerously-skip-permissions" not in command


def test_get_done_task_returns_last_output(client):
    """Done tasks should return saved last_output instead of empty output."""
    import time
    create = client.post("/tasks", json={"prompt": "echo done-output-test"})
    task_id = create.json()["id"]
    time.sleep(1)
    # Kill the task (saves snapshot)
    client.delete(f"/tasks/{task_id}")
    # Fetch the done task — should have output
    r = client.get(f"/tasks/{task_id}")
    assert r.status_code == 200
    assert "done-output-test" in r.json().get("output", "")


def test_hide_task(client):
    create = client.post("/tasks", json={"prompt": "echo hide-api"})
    task_id = create.json()["id"]
    client.delete(f"/tasks/{task_id}")
    r = client.post(f"/tasks/{task_id}/hide")
    assert r.status_code == 200
    assert r.json()["status"] == "hidden"


def test_hide_running_task_fails(client):
    create = client.post("/tasks", json={"prompt": "sleep 999"})
    task_id = create.json()["id"]
    r = client.post(f"/tasks/{task_id}/hide")
    assert r.status_code == 400


def test_hide_nonexistent_task(client):
    r = client.post("/tasks/task-nonexistent/hide")
    assert r.status_code == 404


# Worktree API tests


def test_list_worktrees_no_git_repo(client):
    """List worktrees should return empty list when not in a git repo."""
    from unittest.mock import patch
    from onako import server

    with patch.object(server, "worktree_manager", None):
        r = client.get("/worktrees")
        assert r.status_code == 200
        assert r.json() == []


def test_list_worktrees_with_worktrees(client):
    """List worktrees should return worktree info when manager is available."""
    from unittest.mock import patch, MagicMock
    from pathlib import Path
    from onako import server

    mock_manager = MagicMock()
    mock_manager.list_worktrees.return_value = [
        {"branch": "feature-a", "path": Path("/repo--feature-a")},
        {"branch": "feature-b", "path": Path("/repo--feature-b")},
    ]
    with patch.object(server, "worktree_manager", mock_manager):
        r = client.get("/worktrees")
        assert r.status_code == 200
        result = r.json()
        assert len(result) == 2
        assert result[0]["branch"] == "feature-a"
        assert result[0]["path"] == "/repo--feature-a"
        assert result[1]["branch"] == "feature-b"


def test_delete_worktree_no_git_repo(client):
    """Delete worktree should return 400 when not in a git repo."""
    from unittest.mock import patch
    from onako import server

    with patch.object(server, "worktree_manager", None):
        r = client.delete("/worktrees/feature-a")
        assert r.status_code == 400
        assert "Not in a git repository" in r.json()["detail"]


def test_delete_worktree_success(client):
    """Delete worktree should call delete_worktree on the manager."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_manager = MagicMock()
    with patch.object(server, "worktree_manager", mock_manager):
        r = client.delete("/worktrees/feature-a")
        assert r.status_code == 200
        assert r.json()["status"] == "deleted"
        mock_manager.delete_worktree.assert_called_once_with("feature-a")


def test_delete_worktree_with_slash_in_branch(client):
    """Delete worktree should handle branches with slashes."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_manager = MagicMock()
    with patch.object(server, "worktree_manager", mock_manager):
        r = client.delete("/worktrees/feature/nested/branch")
        assert r.status_code == 200
        mock_manager.delete_worktree.assert_called_once_with("feature/nested/branch")


def test_create_task_with_branch(client):
    """Create task with branch should use worktree and pass branch to orchestrator."""
    from unittest.mock import patch, MagicMock
    from pathlib import Path
    from onako import server

    mock_manager = MagicMock()
    mock_manager.ensure_worktree.return_value = Path("/repo--my-feature")

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})

    with patch.object(server, "worktree_manager", mock_manager), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "implement feature", "branch": "my-feature"})
        assert r.status_code == 200
        mock_manager.ensure_worktree.assert_called_once_with("my-feature", "main")
        # Check that orchestrator was called with branch and worktree_path
        call_kwargs = mock_create.call_args[1]
        assert call_kwargs["working_dir"] == "/repo--my-feature"
        assert call_kwargs["branch"] == "my-feature"
        assert call_kwargs["worktree_path"] == "/repo--my-feature"


def test_create_task_with_branch_and_base(client):
    """Create task with branch and base_branch should pass both to ensure_worktree."""
    from unittest.mock import patch, MagicMock
    from pathlib import Path
    from onako import server

    mock_manager = MagicMock()
    mock_manager.ensure_worktree.return_value = Path("/repo--my-feature")

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})

    with patch.object(server, "worktree_manager", mock_manager), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={
            "prompt": "implement feature",
            "branch": "my-feature",
            "base_branch": "develop"
        })
        assert r.status_code == 200
        mock_manager.ensure_worktree.assert_called_once_with("my-feature", "develop")


def test_create_task_with_branch_no_worktree_manager(client):
    """Create task with branch should ignore branch when no worktree manager."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})

    with patch.object(server, "worktree_manager", None), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={"prompt": "implement feature", "branch": "my-feature"})
        assert r.status_code == 200
        # Should still work, but branch/worktree_path should be None
        call_kwargs = mock_create.call_args[1]
        assert call_kwargs.get("branch") is None
        assert call_kwargs.get("worktree_path") is None


def test_create_task_with_working_dir_and_branch(client):
    """When both working_dir and branch provided, branch worktree should take precedence."""
    from unittest.mock import patch, MagicMock
    from pathlib import Path
    from onako import server

    mock_manager = MagicMock()
    mock_manager.ensure_worktree.return_value = Path("/repo--my-feature")

    mock_create = MagicMock(return_value={"id": "task-fake", "status": "running"})

    with patch.object(server, "worktree_manager", mock_manager), \
         patch.object(server.orch, "create_task", mock_create):
        r = client.post("/tasks", json={
            "prompt": "implement feature",
            "branch": "my-feature",
            "working_dir": "/some/other/dir"
        })
        assert r.status_code == 200
        # Worktree path should override working_dir
        call_kwargs = mock_create.call_args[1]
        assert call_kwargs["working_dir"] == "/repo--my-feature"


def test_branch_worktree_integration(client, tmp_path):
    """Full integration: create task with branch, verify worktree."""
    import os

    # Create a test git repo
    repo = tmp_path / "test-repo"
    repo.mkdir()
    subprocess.run(["git", "init"], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "config", "user.email", "test@test.com"], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "config", "user.name", "Test"], cwd=repo, check=True, capture_output=True)
    (repo / "README.md").write_text("# Test")
    subprocess.run(["git", "add", "."], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "commit", "-m", "initial"], cwd=repo, check=True, capture_output=True)

    # Patch worktree_manager to use test repo
    from onako import server
    from onako.worktree_manager import WorktreeManager
    original_wm = server.worktree_manager
    server.worktree_manager = WorktreeManager(repo)

    try:
        # Create task with branch
        r = client.post("/tasks", json={"prompt": "echo test", "branch": "test-feature"})
        assert r.status_code == 200
        task = r.json()
        assert task["branch"] == "test-feature"
        assert "test-repo--test-feature" in task["worktree_path"]

        # Verify worktree was created
        r = client.get("/worktrees")
        assert r.status_code == 200
        worktrees = r.json()
        assert any(wt["branch"] == "test-feature" for wt in worktrees)
    finally:
        server.worktree_manager = original_wm
        subprocess.run(["git", "worktree", "prune"], cwd=repo, capture_output=True)


def test_stats_endpoint(client):
    """Stats endpoint should return valid stats structure."""
    # Create a couple tasks to generate events
    client.post("/tasks", json={"prompt": "echo stats-test-1"})
    client.post("/tasks", json={"prompt": "echo stats-test-2"})

    r = client.get("/stats")
    assert r.status_code == 200
    stats = r.json()

    # Check structure
    assert "volume" in stats
    assert "records" in stats
    assert "trends" in stats
    assert "heatmap" in stats
    assert "activity_by_hour" in stats

    # Volume should reflect created tasks (at least 2, maybe more from onako-main)
    assert stats["volume"]["total_tasks"] >= 2
    assert len(stats["activity_by_hour"]) == 24


def test_stats_message_tracking(client):
    """Stats should count messages sent."""
    create = client.post("/tasks", json={"prompt": "cat"})
    task_id = create.json()["id"]
    client.post(f"/tasks/{task_id}/message", json={"message": "hello"})
    client.post(f"/tasks/{task_id}/message", json={"message": "world"})

    r = client.get("/stats")
    stats = r.json()
    assert stats["volume"]["total_messages_sent"] >= 2


def test_resume_passes_worktree_manager(client):
    """Resume endpoint passes worktree_manager to orchestrator."""
    from unittest.mock import patch, MagicMock
    from onako import server

    mock_resume = MagicMock(return_value={"id": "task-fake", "status": "running", "prompt": "(resumed) test"})
    server.orch.tasks["task-fake"] = {"id": "task-fake", "status": "done", "prompt": "test"}

    with patch.object(server.orch, "resume_task", mock_resume):
        r = client.post("/tasks/task-fake/resume")
        assert r.status_code == 200
        # Verify worktree_manager was passed
        call_kwargs = mock_resume.call_args
        assert "worktree_manager" in call_kwargs.kwargs or len(call_kwargs.args) >= 3 or \
               call_kwargs[1].get("worktree_manager") is not None



# tests/test_worktree_manager.py
import subprocess
from pathlib import Path
import pytest
from onako.worktree_manager import WorktreeManager


@pytest.fixture
def git_repo(tmp_path):
    """Create a test git repo with initial commit."""
    repo = tmp_path / "repo"
    repo.mkdir()
    subprocess.run(["git", "init"], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "config", "user.email", "test@test.com"], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "config", "user.name", "Test"], cwd=repo, check=True, capture_output=True)
    (repo / "README.md").write_text("# Test")
    subprocess.run(["git", "add", "."], cwd=repo, check=True, capture_output=True)
    subprocess.run(["git", "commit", "-m", "initial"], cwd=repo, check=True, capture_output=True)
    yield repo
    # Cleanup worktrees
    subprocess.run(["git", "worktree", "prune"], cwd=repo, capture_output=True)


@pytest.fixture
def manager(git_repo):
    return WorktreeManager(repo_path=git_repo)


def test_ensure_worktree_creates_branch_and_worktree(manager, git_repo):
    path = manager.ensure_worktree("feature-x")

    # Worktree path is sibling with double-dash separator
    expected = git_repo.parent / f"{git_repo.name}--feature-x"
    assert path == expected
    assert expected.exists()

    # Branch was created
    result = subprocess.run(
        ["git", "branch", "--list", "feature-x"],
        cwd=git_repo, capture_output=True, text=True
    )
    assert "feature-x" in result.stdout


def test_ensure_worktree_reuses_existing(manager, git_repo):
    path1 = manager.ensure_worktree("feature-y")
    path2 = manager.ensure_worktree("feature-y")
    assert path1 == path2


def test_list_worktrees(manager, git_repo):
    manager.ensure_worktree("branch-a")
    manager.ensure_worktree("branch-b")

    worktrees = manager.list_worktrees()

    assert len(worktrees) == 2
    branches = [wt["branch"] for wt in worktrees]
    assert "branch-a" in branches
    assert "branch-b" in branches
    assert all("path" in wt for wt in worktrees)


def test_delete_worktree(manager, git_repo):
    path = manager.ensure_worktree("to-delete")
    assert path.exists()

    manager.delete_worktree("to-delete")

    assert not path.exists()
    worktrees = manager.list_worktrees()
    assert not any(wt["branch"] == "to-delete" for wt in worktrees)



from unittest.mock import patch
from pathlib import Path

import pytest
from click.testing import CliRunner
from onako.cli import main


@pytest.fixture(autouse=True)
def isolate_pid_file(tmp_path):
    """Prevent tests from touching the real PID file."""
    fake_pid = tmp_path / "onako.pid"
    fake_dev = tmp_path / "onako.dev"
    with patch("onako.cli.PID_FILE", fake_pid), \
         patch("onako.cli.DEV_MARKER", fake_dev):
        yield fake_pid


def test_status_reports_something():
    runner = CliRunner()
    result = runner.invoke(main, ["status"])
    assert result.exit_code == 0
    assert "Onako server:" in result.output


def test_stop_completes():
    runner = CliRunner()
    result = runner.invoke(main, ["stop"])
    assert result.exit_code == 0
    assert "not running" in result.output or "stopped" in result.output


def test_stop_handles_no_service():
    """Stop should handle the case where neither pid file nor service exists."""
    runner = CliRunner()
    result = runner.invoke(main, ["stop"])
    assert result.exit_code == 0



import sqlite3
from pathlib import Path
from unittest.mock import patch

import pytest


@pytest.fixture
def db_path(tmp_path):
    return tmp_path / "test.db"


@pytest.fixture
def make_orchestrator(db_path):
    def _make():
        with patch("onako.tmux_orchestrator.TmuxOrchestrator._ensure_session"), \
             patch("onako.tmux_orchestrator.TmuxOrchestrator.rediscover_tasks"):
            from onako.tmux_orchestrator import TmuxOrchestrator
            orch = TmuxOrchestrator(session_name="test", db_path=db_path)
            return orch
    return _make


def test_get_stats_empty(make_orchestrator):
    """Stats with no events should return zero volumes."""
    orch = make_orchestrator()
    stats = orch.get_stats()
    assert stats["volume"]["total_tasks"] == 0
    assert stats["volume"]["total_messages_sent"] == 0
    assert stats["volume"]["total_resumes"] == 0
    assert stats["volume"]["total_kills"] == 0
    assert stats["volume"]["total_interrupts"] == 0
    assert stats["volume"]["total_runtime_seconds"] == 0


def test_get_stats_volume_counts(make_orchestrator, db_path):
    """Stats should count events by type."""
    orch = make_orchestrator()
    conn = sqlite3.connect(db_path)
    events = [
        ("task_created", "t1", "2026-01-15T10:00:00+00:00"),
        ("task_created", "t2", "2026-01-15T11:00:00+00:00"),
        ("task_completed", "t1", "2026-01-15T12:00:00+00:00"),
        ("message_sent", "t2", "2026-01-15T11:05:00+00:00"),
        ("message_sent", "t2", "2026-01-15T11:10:00+00:00"),
        ("task_resumed", "t1", "2026-01-16T10:00:00+00:00"),
        ("task_killed", "t2", "2026-01-16T12:00:00+00:00"),
        ("task_interrupted", "t1", "2026-01-15T10:30:00+00:00"),
    ]
    for et, tid, ts in events:
        conn.execute(
            "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
            (et, tid, ts),
        )
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    assert stats["volume"]["total_tasks"] == 2
    assert stats["volume"]["total_messages_sent"] == 2
    assert stats["volume"]["total_resumes"] == 1
    assert stats["volume"]["total_kills"] == 1
    assert stats["volume"]["total_interrupts"] == 1


def test_get_stats_runtime(make_orchestrator, db_path):
    """Runtime should be computed from task_created to task_completed pairs."""
    orch = make_orchestrator()
    conn = sqlite3.connect(db_path)
    conn.execute(
        "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
        ("task_created", "t1", "2026-01-15T10:00:00+00:00"),
    )
    conn.execute(
        "INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
        ("task_completed", "t1", "2026-01-15T12:00:00+00:00"),
    )
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    assert stats["volume"]["total_runtime_seconds"] == 7200


def test_get_stats_records(make_orchestrator, db_path):
    """Records should find longest task and most resumed task."""
    orch = make_orchestrator()
    orch.tasks["t1"] = {"id": "t1", "prompt": "short task", "status": "done", "started_at": None, "origin": "managed", "last_activity": None, "claude_session_id": None, "branch": None, "worktree_path": None}
    orch.tasks["t2"] = {"id": "t2", "prompt": "long task", "status": "done", "started_at": None, "origin": "managed", "last_activity": None, "claude_session_id": None, "branch": None, "worktree_path": None}

    conn = sqlite3.connect(db_path)
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t1", "2026-01-15T10:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_completed", "t1", "2026-01-15T11:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t2", "2026-01-15T10:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_completed", "t2", "2026-01-15T13:00:00+00:00"))
    for i in range(3):
        conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                     ("task_resumed", "t1", f"2026-01-16T{10+i}:00:00+00:00"))
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    assert stats["records"]["longest_task"]["id"] == "t2"
    assert stats["records"]["longest_task"]["duration_seconds"] == 10800
    assert stats["records"]["most_resumed_task"]["id"] == "t1"
    assert stats["records"]["most_resumed_task"]["resume_count"] == 3


def test_get_stats_heatmap(make_orchestrator, db_path):
    """Heatmap should count events per day."""
    orch = make_orchestrator()
    conn = sqlite3.connect(db_path)
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t1", "2026-01-15T10:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("message_sent", "t1", "2026-01-15T11:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t2", "2026-01-16T10:00:00+00:00"))
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    assert stats["heatmap"].get("2026-01-15") == 2
    assert stats["heatmap"].get("2026-01-16") == 1


def test_get_stats_activity_by_hour(make_orchestrator, db_path):
    """Activity by hour should bucket events into 24 slots."""
    orch = make_orchestrator()
    conn = sqlite3.connect(db_path)
    for i in range(3):
        conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                     ("task_created", f"t{i}", f"2026-01-15T10:{i:02d}:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("message_sent", "t1", "2026-01-15T14:00:00+00:00"))
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    assert len(stats["activity_by_hour"]) == 24
    assert stats["activity_by_hour"][10] == 3
    assert stats["activity_by_hour"][14] == 1
    assert stats["activity_by_hour"][0] == 0


def test_get_stats_trends(make_orchestrator, db_path):
    """Trends should group tasks and messages per ISO week."""
    orch = make_orchestrator()
    conn = sqlite3.connect(db_path)
    # Week 2026-W03 (Jan 12-18)
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t1", "2026-01-15T10:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t2", "2026-01-16T10:00:00+00:00"))
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("message_sent", "t1", "2026-01-15T11:00:00+00:00"))
    # Week 2026-W04 (Jan 19-25)
    conn.execute("INSERT INTO events (event_type, task_id, timestamp) VALUES (?, ?, ?)",
                 ("task_created", "t3", "2026-01-20T10:00:00+00:00"))
    conn.commit()
    conn.close()

    stats = orch.get_stats()
    tasks_per_week = {w["week"]: w["count"] for w in stats["trends"]["tasks_per_week"]}
    assert tasks_per_week.get("2026-W03") == 2
    assert tasks_per_week.get("2026-W04") == 1



from unittest.mock import patch, MagicMock

from click.testing import CliRunner
from onako.cli import main


def test_version():
    runner = CliRunner()
    result = runner.invoke(main, ["version"])
    assert result.exit_code == 0
    assert "onako" in result.output


def test_default_help():
    runner = CliRunner()
    result = runner.invoke(main, ["--help"])
    assert result.exit_code == 0
    assert "--session" in result.output
    assert "--host" in result.output
    assert "--port" in result.output


def test_serve_help():
    runner = CliRunner()
    result = runner.invoke(main, ["serve", "--help"])
    assert result.exit_code == 0
    assert "--host" in result.output
    assert "--port" in result.output
    assert "--session" in result.output
    assert "--dir" in result.output


def test_skip_permissions_flag_in_help():
    runner = CliRunner()
    result = runner.invoke(main, ["--help"])
    assert "--dangerously-skip-permissions" in result.output


def test_serve_skip_permissions_flag_in_help():
    runner = CliRunner()
    result = runner.invoke(main, ["serve", "--help"])
    assert "--dangerously-skip-permissions" in result.output


def test_task_help():
    runner = CliRunner()
    result = runner.invoke(main, ["task", "--help"])
    assert result.exit_code == 0
    assert "PROMPT" in result.output
    assert "--dir" in result.output
    assert "--dangerously-skip-permissions" in result.output
    assert "--branch" in result.output
    assert "--base-branch" in result.output


def test_task_requires_prompt():
    runner = CliRunner()
    result = runner.invoke(main, ["task"])
    assert result.exit_code != 0
    assert "Missing argument" in result.output


def test_task_creates_task_with_mock_orch():
    """Test that `onako task` calls the service and prints the task ID."""
    mock_create = MagicMock(return_value={"id": "task-test123", "status": "running"})

    with patch("onako.cli.subprocess") as mock_subprocess:
        # Make tmux session check pass (has-session returns 0)
        mock_subprocess.run.return_value = MagicMock(returncode=0, stdout="onako\n")
        with patch("onako.tmux_orchestrator.TmuxOrchestrator") as mock_orch_cls:
            with patch("onako.task_service.create_task", mock_create):
                runner = CliRunner()
                result = runner.invoke(main, ["task", "fix the bug"])

    assert result.exit_code == 0, f"CLI failed with: {result.output}"
    assert "task-test123" in result.output
    assert "fix the bug" in result.output


def test_clean_help():
    runner = CliRunner()
    result = runner.invoke(main, ["clean", "--help"])
    assert result.exit_code == 0
    assert "worktree" in result.output.lower()


def test_clean_removes_finished_worktrees(tmp_path):
    """Clean removes worktrees for done/hidden tasks."""
    import sqlite3

    db_path = tmp_path / "onako.db"
    conn = sqlite3.connect(db_path)
    conn.execute("""
        CREATE TABLE tasks (
            id TEXT PRIMARY KEY, prompt TEXT, status TEXT, started_at TEXT,
            origin TEXT, last_activity TEXT, claude_session_id TEXT,
            branch TEXT, worktree_path TEXT, last_output TEXT
        )
    """)
    # A done task with a worktree
    wt_dir = tmp_path / "repo--feat-done"
    wt_dir.mkdir()
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-done1", "prompt", "done", None, "managed", None, None, "feat/done", str(wt_dir), None),
    )
    # A running task with a worktree (should NOT be cleaned)
    wt_running = tmp_path / "repo--feat-running"
    wt_running.mkdir()
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-run1", "prompt", "running", None, "managed", None, None, "feat/running", str(wt_running), None),
    )
    conn.commit()
    conn.close()

    runner = CliRunner()
    with patch("onako.cli.ONAKO_DIR", tmp_path), \
         patch("onako.cli._clean_worktree") as mock_clean:
        result = runner.invoke(main, ["clean"])

    assert result.exit_code == 0
    # Should only clean the done task's worktree
    mock_clean.assert_called_once()
    assert "repo--feat-done" in mock_clean.call_args[0][0]


def test_clean_nothing_to_clean(tmp_path):
    """Clean prints message when no worktrees to clean."""
    import sqlite3

    db_path = tmp_path / "onako.db"
    conn = sqlite3.connect(db_path)
    conn.execute("""
        CREATE TABLE tasks (
            id TEXT PRIMARY KEY, prompt TEXT, status TEXT, started_at TEXT,
            origin TEXT, last_activity TEXT, claude_session_id TEXT,
            branch TEXT, worktree_path TEXT, last_output TEXT
        )
    """)
    conn.commit()
    conn.close()

    runner = CliRunner()
    with patch("onako.cli.ONAKO_DIR", tmp_path):
        result = runner.invoke(main, ["clean"])

    assert result.exit_code == 0
    assert "No worktrees to clean up" in result.output


def test_purge_removed():
    """The purge command should no longer exist."""
    runner = CliRunner()
    result = runner.invoke(main, ["purge"])
    assert result.exit_code != 0


def test_reset_help():
    runner = CliRunner()
    result = runner.invoke(main, ["reset", "--help"])
    assert result.exit_code == 0


def test_reset_stops_server_and_cleans_db(tmp_path):
    """Reset marks running tasks as done and removes all worktrees."""
    import sqlite3

    db_path = tmp_path / "onako.db"
    conn = sqlite3.connect(db_path)
    conn.execute("""
        CREATE TABLE tasks (
            id TEXT PRIMARY KEY, prompt TEXT, status TEXT, started_at TEXT,
            origin TEXT, last_activity TEXT, claude_session_id TEXT,
            branch TEXT, worktree_path TEXT, last_output TEXT
        )
    """)
    wt1 = tmp_path / "repo--feat-a"
    wt1.mkdir()
    wt2 = tmp_path / "repo--feat-b"
    wt2.mkdir()
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-1", "prompt", "running", None, "managed", None, None, "feat/a", str(wt1), None),
    )
    conn.execute(
        "INSERT INTO tasks VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
        ("task-2", "prompt", "done", None, "managed", None, None, "feat/b", str(wt2), None),
    )
    conn.commit()
    conn.close()

    # No PID file, no tmux session — just test DB + worktree cleanup
    pid_file = tmp_path / "onako.pid"

    runner = CliRunner()
    with patch("onako.cli.ONAKO_DIR", tmp_path), \
         patch("onako.cli.PID_FILE", pid_file), \
         patch("onako.cli.DEV_MARKER", tmp_path / "onako.dev"), \
         patch("onako.cli.subprocess") as mock_sub:
        mock_sub.run.return_value = MagicMock(returncode=1, stdout="", stderr="")
        with patch("onako.cli._clean_worktree") as mock_clean:
            result = runner.invoke(main, ["reset"])

    assert result.exit_code == 0
    # DB should have tasks marked as done
    conn = sqlite3.connect(db_path)
    statuses = conn.execute("SELECT status FROM tasks").fetchall()
    conn.close()
    assert all(s[0] == "done" for s in statuses)
    # Both worktrees should be cleaned (reset cleans ALL, not just finished)
    assert mock_clean.call_count == 2



from pathlib import Path
from unittest.mock import MagicMock
from onako.task_service import create_task


def test_create_task_builds_command_without_skip():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}

    result = create_task(orch, prompt="echo hello", skip_permissions=False)

    command = orch.create_task.call_args[1]["command"]
    assert command.startswith("claude --session-id ")
    assert command.endswith("'echo hello'")
    assert "--dangerously-skip-permissions" not in command
    assert result["id"] == "task-abc"


def test_create_task_builds_command_with_skip():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}

    result = create_task(orch, prompt="echo hello", skip_permissions=True)

    command = orch.create_task.call_args[1]["command"]
    assert command.startswith("claude --dangerously-skip-permissions --session-id ")
    assert command.endswith("'echo hello'")
    assert result["id"] == "task-abc"


def test_create_task_passes_working_dir():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}

    create_task(orch, prompt="echo hello", working_dir="/tmp/mydir")

    assert orch.create_task.call_args[1]["working_dir"] == "/tmp/mydir"


def test_create_task_passes_prompt_and_session_id():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}

    create_task(orch, prompt="fix the bug")

    kwargs = orch.create_task.call_args[1]
    assert kwargs["prompt"] == "fix the bug"
    assert kwargs["claude_session_id"] is not None


def test_create_task_with_branch_and_worktree_manager():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}
    wt_mgr = MagicMock()
    wt_mgr.ensure_worktree.return_value = Path("/repos/myproject--feat-auth")

    create_task(orch, prompt="add auth", branch="feat/auth", worktree_manager=wt_mgr)

    wt_mgr.ensure_worktree.assert_called_once_with("feat/auth", "main")
    kwargs = orch.create_task.call_args[1]
    assert kwargs["working_dir"] == "/repos/myproject--feat-auth"
    assert kwargs["branch"] == "feat/auth"
    assert kwargs["worktree_path"] == "/repos/myproject--feat-auth"


def test_create_task_with_branch_and_custom_base():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}
    wt_mgr = MagicMock()
    wt_mgr.ensure_worktree.return_value = Path("/repos/myproject--feat-auth")

    create_task(orch, prompt="add auth", branch="feat/auth", base_branch="develop", worktree_manager=wt_mgr)

    wt_mgr.ensure_worktree.assert_called_once_with("feat/auth", "develop")


def test_create_task_branch_without_worktree_manager_ignored():
    orch = MagicMock()
    orch.create_task.return_value = {"id": "task-abc", "status": "running"}

    create_task(orch, prompt="add auth", branch="feat/auth")

    kwargs = orch.create_task.call_args[1]
    assert kwargs["branch"] is None
    assert kwargs["worktree_path"] is None



import shlex
import subprocess
import uuid

from onako.tmux_orchestrator import TmuxOrchestrator


def create_task(
    orch: TmuxOrchestrator,
    prompt: str,
    working_dir: str | None = None,
    skip_permissions: bool = False,
    branch: str | None = None,
    base_branch: str | None = None,
    worktree_manager=None,
) -> dict:
    """Create a new Claude Code task in a tmux window.

    Shared by the API endpoint and CLI command.
    """
    claude_session_id = str(uuid.uuid4())
    if skip_permissions:
        command = f"claude --dangerously-skip-permissions --session-id {claude_session_id} {shlex.quote(prompt)}"
    else:
        command = f"claude --session-id {claude_session_id} {shlex.quote(prompt)}"

    worktree_path = None
    if branch and worktree_manager:
        base = base_branch or "main"
        wt_path = worktree_manager.ensure_worktree(branch, base)
        working_dir = str(wt_path)
        worktree_path = str(wt_path)

    return orch.create_task(
        command=command,
        working_dir=working_dir,
        prompt=prompt,
        claude_session_id=claude_session_id,
        branch=branch if worktree_manager else None,
        worktree_path=worktree_path,
    )
